@@ -39,7 +39,8 @@ import Data.X509.Validation.Signature
 import Data.X509.Validation.Fingerprint
 import Data.X509.Validation.Cache
 import Data.X509.Validation.Types
-import Data.Time.Clock
+import Data.Hourglass
+import System.Hourglass
 import Data.Maybe
 import Data.List
 
@@ -77,7 +78,7 @@ data ValidationChecks = ValidationChecks
       checkTimeValidity   :: Bool
     -- | The time when the validity check happens. When set to Nothing,
     -- the current time will be used
-    , checkAtTime         :: Maybe UTCTime
+    , checkAtTime         :: Maybe DateTime
     -- | Check that no certificate is included that shouldn't be included.
     -- unfortunately despite the specification violation, a lots of
     -- real world server serves useless and usually old certificates
@@ -120,7 +121,7 @@ data ValidationHooks = ValidationHooks
     -- of a certificate.
       hookMatchSubjectIssuer :: DistinguishedName -> Certificate -> Bool
     -- | validate that the parametrized time valide with the certificate in argument
-    , hookValidateTime       :: UTCTime -> Certificate -> [FailedReason]
+    , hookValidateTime       :: DateTime -> Certificate -> [FailedReason]
     -- | validate the certificate leaf name with the DNS named used to connect
     , hookValidateName       :: HostName -> Certificate -> [FailedReason]
     -- | user filter to modify the list of failure reasons
@@ -190,7 +191,7 @@ validate hashAlg hooks checks store cache ident cc@(CertificateChain (top:_)) =
         ValidationCachePass     -> return []
         ValidationCacheDenied s -> return [CacheSaysNo s]
         ValidationCacheUnknown  -> do
-            validationTime <- maybe getCurrentTime return $ checkAtTime checks
+            validationTime <- maybe (timeConvert <$> timeCurrent) return $ checkAtTime checks
             failedReasons <- doValidate validationTime hooks checks store ident cc
             when (null failedReasons) $ (cacheAdd cache) ident fingerPrint (getCertificate top)
             return failedReasons
@@ -198,7 +199,7 @@ validate hashAlg hooks checks store cache ident cc@(CertificateChain (top:_)) =
 
 
 -- | Validate a certificate chain with explicit parameters
-doValidate :: UTCTime
+doValidate :: DateTime
            -> ValidationHooks
            -> ValidationChecks
            -> CertificateStore
@@ -308,7 +309,7 @@ doValidate validationTime hooks checks store (fqhn,_) (CertificateChain (top:rch
                 SignatureFailed r -> [InvalidSignature r]
 
 -- | Validate that the current time is between validity bounds
-validateTime :: UTCTime -> Certificate -> [FailedReason]
+validateTime :: DateTime -> Certificate -> [FailedReason]
 validateTime currentTime cert
     | currentTime < before = [InFuture]
     | currentTime > after  = [Expired]
@@ -324,13 +325,16 @@ getNames cert = (commonName >>= asn1CharacterToString, altNames)
                   unAltName _              = Nothing
 
 -- | Validate that the fqhn is matched by at least one name in the certificate.
--- The name can be either the common name or one of the alternative names if
--- the SubjectAltName extension is present.
+-- The name can be either one of the alternative names if the SubjectAltName
+-- extension is present or the common name.
 validateCertificateName :: HostName -> Certificate -> [FailedReason]
-validateCertificateName fqhn cert =
-    case commonName of
-        Nothing -> [NoCommonName]
-        Just cn -> findMatch [] $ map (matchDomain . splitDot) (cn : altNames)
+validateCertificateName fqhn cert
+    | not $ null altNames =
+        findMatch [] $ map (matchDomain . splitDot) altNames
+    | otherwise =
+        case commonName of
+            Nothing -> [NoCommonName]
+            Just cn -> findMatch [] $ [matchDomain $ splitDot $ cn]
   where (commonName, altNames) = getNames cert
 
         findMatch :: [FailedReason] -> [[FailedReason]] -> [FailedReason]
@@ -353,7 +357,7 @@ validateCertificateName fqhn cert =
         wildcardMatch l
             -- <star>.com or <star> is always invalid
             | length l < 2 = [InvalidWildcard]
-            -- some TLD like .uk got small subTLS like (.co.uk), and we don't want to accept *.co.uk
+            -- some TLD like .uk got small subTLD like (.co.uk), and we don't want to accept *.co.uk
             | length (head l) <= 2 && length (head $ drop 1 l) <= 3 && length l < 3 = [InvalidWildcard]
             | l == take (length l) (reverse $ splitDot fqhn) = [] -- success: we got a match
             | otherwise                                      = [NameMismatch fqhn]
